@@ -28,12 +28,10 @@ import abc
 from debtcollector import removals
 import netaddr
 import rfc3986
-import six
-from six.moves import range as compat_range
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ConfigType(object):
+class ConfigType(metaclass=abc.ABCMeta):
+
     def __init__(self, type_name='unknown type'):
         self.type_name = type_name
 
@@ -44,7 +42,7 @@ class ConfigType(object):
 
         """
         if sample_default is not None:
-            if isinstance(sample_default, six.string_types):
+            if isinstance(sample_default, str):
                 default_str = sample_default
             else:
                 default_str = self._formatter(sample_default)
@@ -55,9 +53,9 @@ class ConfigType(object):
         return [default_str]
 
     def quote_trailing_and_leading_space(self, str_val):
-        if not isinstance(str_val, six.string_types):
+        if not isinstance(str_val, str):
             warnings.warn('converting \'%s\' to a string' % str_val)
-            str_val = six.text_type(str_val)
+            str_val = str(str_val)
         if str_val.strip() != str_val:
             return '"%s"' % str_val
         return str_val
@@ -135,7 +133,7 @@ class String(ConfigType):
             re_flags = re.IGNORECASE if self.ignore_case else 0
 
             # Check if regex is a string or an already compiled regex
-            if isinstance(regex, six.string_types):
+            if isinstance(regex, str):
                 self.regex = re.compile(regex, re_flags)
             else:
                 self.regex = re.compile(regex.pattern, re_flags | regex.flags)
@@ -144,7 +142,7 @@ class String(ConfigType):
         value = str(value)
         if self.quotes and value:
             if value[0] in "\"'":
-                if value[-1] != value[0]:
+                if len(value) == 1 or value[-1] != value[0]:
                     raise ValueError('Non-closed quote: %s' % value)
                 value = value[1:-1]
 
@@ -351,7 +349,7 @@ class Number(ConfigType):
         )
 
     def _formatter(self, value):
-        return six.text_type(value)
+        return str(value)
 
 
 class Integer(Number):
@@ -484,7 +482,7 @@ class List(ConfigType):
 
     def __call__(self, value):
         if isinstance(value, (list, tuple)):
-            return list(six.moves.map(self.item_type, value))
+            return list(map(self.item_type, value))
 
         s = value.strip().rstrip(',')
         if self.bounds:
@@ -531,7 +529,7 @@ class List(ConfigType):
 
     def _formatter(self, value):
         fmtstr = '[{}]' if self.bounds else '{}'
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             return fmtstr.format(value)
         if isinstance(value, list):
             value = [
@@ -585,7 +583,7 @@ class Range(ConfigType):
             step = -1
         if self.inclusive:
             right += step
-        return compat_range(left, right, step)
+        return range(left, right, step)
 
     def __eq__(self, other):
         return (
@@ -763,11 +761,12 @@ class Hostname(ConfigType):
     :param type_name: Type name to be used in the sample config file.
 
     """
+    HOSTNAME_REGEX = '(?!-)[A-Z0-9-]{1,63}(?<!-)$'
 
     def __init__(self, type_name='hostname value'):
         super(Hostname, self).__init__(type_name=type_name)
 
-    def __call__(self, value):
+    def __call__(self, value, regex=HOSTNAME_REGEX):
         """Check hostname is valid.
 
         Ensures that each segment
@@ -791,7 +790,7 @@ class Hostname(ConfigType):
                              % value)
         if value.endswith("."):
             value = value[:-1]
-        allowed = re.compile("(?!-)[A-Z0-9-]{1,63}(?<!-)$", re.IGNORECASE)
+        allowed = re.compile(regex, re.IGNORECASE)
         if not re.search('[a-zA-Z-]', value.split(".")[-1]):
             raise ValueError('%s contains no non-numeric characters in the '
                              'top-level domain part of the host name and is '
@@ -846,7 +845,8 @@ class HostAddress(ConfigType):
             try:
                 value = self.hostname(value)
             except ValueError:
-                raise ValueError("%s is not a valid host address" % (value,))
+                raise ValueError(
+                    "%s is not a valid host address" % (value,))
         return value
 
     def __repr__(self):
@@ -859,6 +859,52 @@ class HostAddress(ConfigType):
         return value
 
 
+class HostDomain(HostAddress):
+    """Host Domain type.
+
+    Like HostAddress with the support of _ character.
+
+    :param version: defines which version should be explicitly
+                    checked (4 or 6) in case of an IP address
+    :param type_name: Type name to be used in the sample config file.
+    """
+    # DOMAIN_REGEX is HOSTNAME_REGEX with the _ character added
+    DOMAIN_REGEX = '(?!-)[A-Z0-9-_]{1,63}(?<!-)$'
+
+    def __init__(self, version=None, type_name='host domain value'):
+        """Check for valid version in case an IP address is provided
+
+        """
+
+        super(HostDomain, self).__init__(type_name=type_name)
+
+    def __call__(self, value):
+        """Checks if is a valid IP/hostname.
+
+        If not a valid IP, makes sure it is not a mistyped IP before
+        performing checks for it as a hostname.
+
+        """
+
+        try:
+            value = super(HostDomain, self).__call__(value)
+        except ValueError:
+            # Check if domain is valid
+            # Add support of underscore
+            # https://www.ietf.org/rfc/rfc1912,
+            # http://domainkeys.sourceforge.net/underscore.html
+            # https://bugs.launchpad.net/oslo.config/+bug/1892044
+            try:
+                value = self.hostname(value, regex=self.DOMAIN_REGEX)
+            except ValueError:
+                raise ValueError(
+                    "%s is not a valid host address" % (value,))
+        return value
+
+    def __repr__(self):
+        return 'HostDomain'
+
+
 class URI(ConfigType):
 
     """URI type
